@@ -1,8 +1,8 @@
-## PGLS.R (2017-04-25)
+## PGLS.R (2021-12-18)
 
 ##   Phylogenetic Generalized Least Squares
 
-## Copyright 2004 Julien Dutheil, and 2006-2017 Emmanuel Paradis
+## Copyright 2004-2021 Julien Dutheil, and 2006-2017 Emmanuel Paradis
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
@@ -44,6 +44,7 @@ corGrafen <- function(value, phy, form = ~1, fixed = FALSE)
     attr(value, "fixed") <- fixed
     attr(value, "tree") <- phy
     class(value) <- c("corGrafen", "corPhyl", "corStruct")
+
     value
 }
 
@@ -51,6 +52,9 @@ Initialize.corPhyl <- function(object, data, ...)
 {
     ## The same as in Initialize corStruct:
     form <- formula(object)
+    if (getCovariateFormula(object) == ~1) {
+      warning("No covariate specified, species will be taken as ordered in the data frame. To avoid this message, specify a covariate containing the species names with the 'form' argument.")
+    }
     ## Obtaining the group information, if any
     if (!is.null(getGroupsFormula(form))) {
         attr(object, "groups") <- getGroups(object, form, data = data)
@@ -61,25 +65,6 @@ Initialize.corPhyl <- function(object, data, ...)
     ## Obtaining the covariate(s)
     attr(object, "covariate") <- getCovariate(object, data = data)
 
-    ## Specific to corPhyl:
-    phy <- attr(object, "tree")
-    if (is.null(data)) data <- parent.frame()
-    ## Added by EP 29 May 2006:
-    if (nrow(data) != length(phy$tip.label))
-        stop("number of observations and number of tips in the tree are not equal.")
-    ## END
-    if (is.null(rownames(data))) {
-        warning("No rownames supplied in data frame, data taken to be in the same order than in tree")
-        attr(object, "index") <- 1:dim(data)[1]
-    } else {
-        index <- match(rownames(data), phy$tip.label)
-        if (any(is.na(index))) {
-            warning("Rownames in data frame do not match tree tip names; data taken to be in the same order as in tree")
-            attr(object, "index") <- 1:dim(data)[1]
-        } else {
-            attr(object, "index") <- index
-        }
-    }
     object
 }
 
@@ -88,13 +73,16 @@ corMatrix.corBrownian <-
 {
     if (!("corBrownian" %in% class(object)))
         stop('object is not of class "corBrownian"')
-    if (!any(attr(object, "index")))
-        stop("object has not been initialized.")
-    tree <- attr(object, "tree")
-    mat <- vcv.phylo(tree, corr = corr)
-    ## reorder matrix:
-    index <- attr(object, "index")
-    mat[index, index]
+    if (data.class(covariate) == "list") {
+        as.list(lapply(covariate, function(el) corMatrix(object, covariate = el)))
+    } else {
+        ##if (! is.character(covariate)) {
+        ##    stop("Covariate must be a character vector.")
+        ##}
+        tree <- attr(object, "tree")
+        mat <- vcv.phylo(tree, corr = corr)
+        mat[covariate, covariate]
+    }
 }
 
 corMatrix.corMartins <-
@@ -102,15 +90,18 @@ corMatrix.corMartins <-
 {
     if (!("corMartins" %in% class(object)))
         stop('object is not of class "corMartins"')
-    if (!any(attr(object, "index")))
-        stop("object has not been initialized.")
-    tree <- attr(object, "tree")
-    dist <- cophenetic.phylo(tree)
-    mat <- exp(-object[1] * dist)
-    if (corr) mat <- cov2cor(mat)
-    ## reorder matrix:
-    index <- attr(object, "index")
-    mat[index, index]
+    if (data.class(covariate) == "list") {
+        as.list(lapply(covariate, function(el) corMatrix(object, covariate = el)))
+    } else {
+        ##if (! is.character(covariate)) {
+        ##    stop("Covariate must be a character vector.")
+        ##}
+        tree <- attr(object, "tree")
+        dist <- cophenetic.phylo(tree)
+        mat <- exp(-object[1] * dist)
+        if (corr) mat <- cov2cor(mat)
+        mat[covariate, covariate]
+    }
 }
 
 corMatrix.corGrafen <-
@@ -118,14 +109,17 @@ corMatrix.corGrafen <-
 {
     if (!("corGrafen" %in% class(object)))
         stop('object is not of class "corGrafen"')
-    if (!any(attr(object, "index")))
-        stop("object has not been initialized.")
-    tree <- compute.brlen(attr(object, "tree"),
-                          method = "Grafen", power = exp(object[1]))
-    mat <- vcv.phylo(tree, corr = corr)
-    ## reorder matrix:
-    index <- attr(object, "index")
-    mat[index, index]
+    if (data.class(covariate) == "list") {
+        as.list(lapply(covariate, function(el) corMatrix(object, covariate = el)))
+    } else {
+        ##if (! is.character(covariate)) {
+        ##    stop("Covariate must be a character vector.")
+        ##}
+        tree <- compute.brlen(attr(object, "tree"),
+                              method = "Grafen", power = exp(object[1]))
+        mat <- vcv.phylo(tree, corr = corr)
+        mat[covariate, covariate]
+    }
 }
 
 coef.corBrownian <- function(object, unconstrained = TRUE, ...)
@@ -214,15 +208,21 @@ corPagel <- function(value, phy, form = ~1, fixed = FALSE)
 corMatrix.corPagel <-
     function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-    if (!any(attr(object, "index")))
-        stop("object has not been initialized")
-    mat <- vcv.phylo(attr(object, "tree"), corr = corr)
-    index <- attr(object, "index")
-    mat <- mat[index, index]
-    tmp <- diag(mat)
-    mat <- object[1]*mat
-    diag(mat) <- tmp
-    mat
+    if (!("corPagel" %in% class(object)))
+        stop('object is not of class "corPagel"')
+    if (data.class(covariate) == "list") {
+        as.list(lapply(covariate, function(el) corMatrix(object, covariate = el)))
+    } else {
+        ##if (! is.character(covariate)) {
+        ##    stop("Covariate must be a character vector.")
+        ##}
+        mat <- vcv.phylo(attr(object, "tree"), corr = corr)
+        mat <- mat[covariate, covariate]
+        tmp <- diag(mat)
+        mat <- object[1]*mat
+        diag(mat) <- tmp
+        mat
+    }
 }
 
 coef.corPagel <- function(object, unconstrained = TRUE, ...)
@@ -252,17 +252,21 @@ corBlomberg <- function(value, phy, form = ~1, fixed = FALSE)
 corMatrix.corBlomberg <-
     function(object, covariate = getCovariate(object), corr = TRUE, ...)
 {
-    index <- attr(object, "index")
-    if (is.null(index))
-        stop("object has not been initialized")
     if (object[1] <= 0)
         stop("the optimization has reached a value <= 0 for parameter 'g':
 probably need to set 'fixed = TRUE' in corBlomberg().")
-    phy <- attr(object, "tree")
-    d <- (dist.nodes(phy)[length(phy$tip.label) + 1, ])^(1/object[1])
-    phy$edge.length <- d[phy$edge[, 2]] - d[phy$edge[, 1]]
-    mat <- vcv.phylo(phy, corr = corr)
-    mat[index, index]
+    if (data.class(covariate) == "list") {
+        as.list(lapply(covariate, function(el) corMatrix(object, covariate = el)))
+    } else {
+        ##if (! is.character(covariate)) {
+        ##    stop("Covariate must be a character vector.")
+        ##}
+        phy <- attr(object, "tree")
+        d <- (dist.nodes(phy)[length(phy$tip.label) + 1, ])^(1/object[1])
+        phy$edge.length <- d[phy$edge[, 2]] - d[phy$edge[, 1]]
+        mat <- vcv.phylo(phy, corr = corr)
+        mat[covariate, covariate]
+    }
 }
 
 coef.corBlomberg <- function(object, unconstrained = TRUE, ...)
