@@ -1,34 +1,44 @@
-## summary.phylo.R (2011-08-04)
+## summary.phylo.R (2021-11-27)
 
-##   Print Summary of a Phylogeny and "multiPhylo" operators
+##   Print Summary of a Phylogeny, "multiPhylo" operators, node degrees
 
-## Copyright 2003-2011 Emmanuel Paradis, and 2006 Ben Bolker
+## Copyright 2003-2021 Emmanuel Paradis, 2006 Ben Bolker, and Klaus Schliep 2016
 
 ## This file is part of the R-package `ape'.
 ## See the file ../COPYING for licensing issues.
 
-Ntip <- function(phy)
+Ntip <- function(phy) UseMethod("Ntip")
+
+Ntip.phylo <- function(phy) length(phy$tip.label)
+
+Ntip.multiPhylo <- function(phy)
 {
-    if (!inherits(phy, "phylo"))
-        stop('object "phy" is not of class "phylo"')
-    length(phy$tip.label)
+    labs <- attr(phy, "TipLabel")
+    if (is.null(labs)) sapply(unclass(phy), Ntip.phylo)
+    else setNames(rep(length(labs), length(phy)), names(phy))
 }
 
-Nnode <- function(phy, internal.only = TRUE)
+Nnode <- function(phy, ...) UseMethod("Nnode")
+
+Nnode.phylo <- function(phy, internal.only = TRUE, ...)
 {
-    if (!inherits(phy, "phylo"))
-        stop('object "phy" is not of class "phylo"')
     if (internal.only) return(phy$Nnode)
     phy$Nnode + length(phy$tip.label)
 }
 
-Nedge <- function(phy)
+Nnode.multiPhylo <- function(phy, internal.only = TRUE, ...)
 {
-    if (!inherits(phy, "phylo"))
-        stop('object "phy" is not of class "phylo"')
-    dim(phy$edge)[1]
+    res <- sapply(unclass(phy), "[[", "Nnode")
+    if (internal.only) return(res)
+    res + Ntip.multiPhylo(phy)
 }
 
+Nedge <- function(phy) UseMethod("Nedge")
+
+Nedge.phylo <- function(phy) dim(phy$edge)[1]
+
+Nedge.multiPhylo <- function(phy) sapply(unclass(phy), Nedge.phylo)
+
 summary.phylo <- function(object, ...)
 {
     cat("\nPhylogenetic tree:", deparse(substitute(object)), "\n\n")
@@ -81,15 +91,17 @@ print.phylo <- function(x, printlen = 6,...)
               "internal nodes.\n\n"))
     cat("Tip labels:\n")
     if (nb.tip > printlen) {
-        cat(paste("\t", paste(x$tip.label[1:printlen],
-                              collapse=", "), ", ...\n", sep = ""))
-    } else print(x$tip.label)
+        cat("  ", paste(x$tip.label[1:printlen], collapse=", "), ", ...\n", sep = "")
+    } else {
+        cat("  ", paste(x$tip.label, collapse=", "), "\n", sep = "")
+    }
     if (!is.null(x$node.label)) {
-        cat("\tNode labels:\n")
+        cat("Node labels:\n")
         if (nb.node > printlen) {
-            cat(paste("\t", paste(x$node.label[1:printlen],
-                                 collapse=", "), ", ...\n", sep = ""))
-        } else print(x$node.label)
+            cat("  ", paste(x$node.label[1:printlen], collapse=", "), ", ...\n", sep = "")
+        } else {
+            cat("  ", paste(x$node.label, collapse=", "), "\n", sep = "")
+        }
     }
     rlab <- if (is.rooted(x)) "Rooted" else "Unrooted"
     cat("\n", rlab, "; ", sep="")
@@ -102,7 +114,7 @@ print.phylo <- function(x, printlen = 6,...)
 print.multiPhylo <- function(x, details = FALSE, ...)
 {
     N <- length(x)
-    cat(N, "phylogenetic trees\n")
+    cat(N, "phylogenetic", ifelse(N > 1, "trees\n", "tree\n"))
     if (details)
       for (i in 1:N)
         cat("tree", i, ":", length(x[[i]]$tip.label), "tips\n")
@@ -134,28 +146,67 @@ str.multiPhylo <- function(object, ...)
     str(object, ...)
 }
 
-c.phylo <- function(..., recursive = FALSE)
-    structure(list(...), class = "multiPhylo")
-## only the first object in '...' is checked for its class,
-## but that should be OK for the moment
+.c_phylo_single <- function(phy) structure(list(phy), class = "multiPhylo")
 
-c.multiPhylo <- function(..., recursive = FALSE)
+c.phylo <- function(..., recursive = TRUE)
 {
     obj <- list(...)
-    n <- length(obj)
-    x <- obj[[1L]]
-    N <- length(x)
-    i <- 2L
-    while (i <= n) {
-        a <- N + 1L
-        N <- N + length(obj[[i]])
-        ## x is of class "multiPhylo", so this uses the operator below:
-        x[a:N] <- obj[[i]]
-        i <- i + 1L
+    classes <- lapply(obj, class)
+    isphylo <- sapply(classes, function(x) "phylo" %in% x)
+    if (all(isphylo)) {
+        class(obj) <- "multiPhylo"
+        return(obj)
     }
+    if (!recursive) return(obj)
+    ismulti <- sapply(classes, function(x) "multiPhylo" %in% x)
+    if (all(isphylo | ismulti)) {
+        for (i in which(isphylo)) obj[[i]] <- .c_phylo_single(obj[[i]])
+        ## added by Klaus:
+        for (i in which(ismulti)) obj[[i]] <- .uncompressTipLabel(obj[[i]])
+        obj <- .makeMultiPhyloFromObj(obj)
+    } else {
+        warning('some objects not of class "phylo" or "multiPhylo": argument recursive=TRUE ignored')
+    }
+    obj
+}
+
+# this is an option to avoid growing the list, better check it also
+# not really as important as long the list of trees is short (by Klaus)
+.makeMultiPhyloFromObj <- function(obj)
+{
+    n <- length(obj)
+    N <- lengths(obj, FALSE)
+    x <- vector("list", sum(N))
+    a <- b <- 0L
+    for (i in 1:n) {
+        a <- b + 1L
+        b <- b + N[i]
+        z <- obj[[i]]
+        x[a:b] <- z
+        if (inherits(z, "multiPhylo") && !is.null(nms <- names(z)))
+            names(x)[a:b] <- nms # see issue #37 on GH
+    }
+    class(x) <- "multiPhylo"
     x
 }
 
+c.multiPhylo <- function(..., recursive = TRUE)
+{
+    obj <- list(...)
+    if (!recursive) return(obj)
+    classes <- lapply(obj, class)
+    isphylo <- sapply(classes, function(x) "phylo" %in% x)
+    ismulti <- sapply(classes, function(x) "multiPhylo" %in% x)
+    if (!all(isphylo | ismulti)) {
+        warning('some objects not of class "phylo" or "multiPhylo": argument recursive=TRUE ignored')
+        return(obj)
+    }
+    for (i in which(isphylo)) obj[[i]] <- .c_phylo_single(obj[[i]])
+    ## added by Klaus
+    for (i in which(ismulti)) obj[[i]] <- .uncompressTipLabel(obj[[i]])
+    .makeMultiPhyloFromObj(obj)
+}
+
 .uncompressTipLabel <- function(x)
 {
     Lab <- attr(x, "TipLabel")
@@ -169,7 +220,9 @@ c.multiPhylo <- function(..., recursive = FALSE)
 
 `[<-.multiPhylo` <- function(x, ..., value)
 {
-    ## recycling is allowed so no need to check: length(value) != length(..1)
+###    ## recycling is allowed so no need to check: length(value) != length(..1)
+###    dots <- list(...)
+###    dots <- if (length(dots)) dots[[1]] else seq_along(x) # see issue #36 on GH
 
     ## check that all elements in 'value' inherit class "phylo"
     test <- unlist(lapply(value, function(xx) !inherits(xx, "phylo")))
@@ -181,14 +234,17 @@ c.multiPhylo <- function(..., recursive = FALSE)
 
     if (is.null(attr(x, "TipLabel"))) {
         x[..1] <- value
+###        x[dots] <- value
         class(x) <- oc
         return(x)
     }
 
     x[..1] <- 0L # in case x needs to be elongated
+###    x[dots] <- 0L # in case x needs to be elongated
     class(x) <- oc
     j <- 1L
     for (i in ..1) {
+###    for (i in dots) {
         ## x is of class "multiPhylo", so this uses the operator below:
         x[[i]] <- value[[j]]
         j <- j + 1L
@@ -229,3 +285,25 @@ c.multiPhylo <- function(..., recursive = FALSE)
     x[[..1]] <- value
     x
 }
+
+degree <- function(x, ...) UseMethod("degree")
+
+degree.phylo <- function(x, details = FALSE, ...)
+{
+    N <- length(x$tip.label) + x$Nnode
+    res <- tabulate(x$edge, N)
+    if (details) return(res)
+    tab <- tabulate(res)
+    DF <- data.frame(Degree = seq_along(tab), Number = tab)
+    DF[tab > 0, ]
+}
+
+degree.evonet <- function(x, details = FALSE, ...)
+{
+    N <- length(x$tip.label) + x$Nnode
+    res <- tabulate(x$edge, N) + tabulate(x$reticulation, N)
+    if (details) return(res)
+    tab <- tabulate(res)
+    DF <- data.frame(Degree = seq_along(tab), Number = tab)
+    DF[tab > 0, ]
+}
